#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of ofunctions module

"""
Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes
"""

__intname__ = "tests.ofunctions.bisection"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2022041501"

from ofunctions.bisection import *


def test_bisect():
    # Define two test functions
    def test_func(i):
        # type: (int) -> bool

        return i >= 1386

    def test_func_multi_arg(i, some_arg=None):
        if some_arg is None:
            assert False, "test_func_multi_arg argument x is None"
        return i >= 1386

    result = bisect(test_func, args_list=range(0, 1600))
    print("Bisect result = %s" % result)
    assert result == 1386, "bisection left to right failed"

    result = bisect(test_func, range(1600, 0, -1))
    print("Bisect result = %s" % result)
    assert result == 1386, "bisection right to left failed"

    result = bisect(test_func, [1, 5, 1300, 1600, 1800, 1900])
    print("Bisect result = %s" % result)
    assert result == 1600, "bisection left to right failed"

    result = bisect(test_func, [1900, 1800, 1600, 1300, 5, 1])
    print("Bisect result = %s" % result)
    assert result == 1600, "bisection right to left failed"

    result = bisect(test_func, [1, 1400, 1700])
    print("Bisect result = %s" % result)
    assert result == 1400, "bisection left to right for 3 args failed"

    result = bisect(test_func, [1, 1300, 1700])
    print("Bisect result = %s" % result)
    assert result == 1700, "bisection left to right for 3 args failed"

    result = bisect(test_func, [1900, 1776, 30])
    print("Bisect result = %s" % result)
    assert result == 1776, "bisection right to left for 3 args failed"

    result = bisect(test_func, [1900, 1200, 30])
    print("Bisect result = %s" % result)
    assert result == 1900, "bisection right to left for 3 args failed"

    result = bisect(test_func, [1, 1500])
    print("Bisect result = %s" % result)
    assert result == 1500, "bisection left to right wfor 2 args failed"

    result = bisect(test_func, [1500, 1000])
    print("Bisect result = %s" % result)
    assert result == 1500, "bisection right to left for 2 args failed"

    try:
        result = bisect(test_func, [1, 1000])
        print("Bisect result = %s" % result)
    except ValueError:
        pass
    else:
        assert (
            False
        ), "bisection left to right without result should produce a ValueError"

    try:
        result = bisect(test_func, [1000, 100])
        print("Bisect result = %s" % result)
    except ValueError:
        pass
    else:
        assert (
            False
        ), "bisection right to left without result should produce a ValueError"

    try:
        result = bisect(test_func, [6000, 9000], allow_all_expected=False)
        print("Bisect result = %s" % result)
    except ValueError:
        pass
    else:
        assert (
            False
        ), "Bisect only valid results with allow_all_expected=False should produce errors"

    result = bisect(test_func, range(100, 1500, 100))
    print("Bisect result = %s" % result)
    assert result == 1400, "bisection left to right for even number of args"

    result = bisect(test_func, range(1500, 100, -100))
    print("Bisect result = %s" % result)
    assert result == 1400, "bisection right to left for even number of args"

    result = bisect(test_func, range(100, 1600, 100))
    print("Bisect result = %s" % result)
    assert result == 1400, "bisection left to right for odd number of args"

    result = bisect(test_func, range(1400, 1000, -100))
    print("Bisect result = %s" % result)
    assert result == 1400, "bisection right to left for odd number of args"

    try:
        result = bisect(test_func, [1])
        print("Bisect result = %s" % result)
    except ValueError:
        pass
    else:
        assert result is None, "bisection with one argument should produce a ValueError"

    # Bisect tests for multiple arguments
    result = bisect(
        test_func_multi_arg, args_list=[(1, "arg2"), (1500, "arg2"), (1800, "arg3")]
    )
    print("Bisect result = {}".format(result))
    assert result[0] == 1500, "bisection for multiple arguments failed"

    result = bisect(
        test_func_multi_arg, args_list=[(x, "somearg") for x in range(0, 10000)]
    )
    print("Bisect result = {}".format(result))
    assert result[0] == 1386, "bisection for multiple arguments failed"

    args = [(x, y) for x, y in zip(range(0, 10000), range(10000, 0, -1))]
    result = bisect(test_func_multi_arg, args_list=args)
    print("Bisect result = {}".format(result))
    assert result[0] == 1386, "bisection for multiple arguments failed"

    result = bisect(test_func, range(0, 5000), expected_result=False)
    print("Bisect result = {}".format(result))
    assert result == 1385, "bisection last false result left to right failed"

    result = bisect(test_func, range(5000, 0, -1), expected_result=False)
    print("Bisect result = {}".format(result))
    assert result == 1385, "bisection last false result right to left failed"

    # Special case when all arguments will be expected results
    result = bisect(test_func, [1386, 1387, 1388], allow_all_expected=True)
    print("Bisect result = {}".format(result))
    assert (
        result == 1388
    ), "bisection with only valid results should return most right result"


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_bisect()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of ofunctions package

"""
ofunctions.mailer is that nice email class that does all the security and bulk sending for you

Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes

"""

__intname__ = "ofunctions.mailer"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2014-2025 Orsiris de Jong"
__description__ = "Mail sending class that handles encryption, authentication, bulk and split mail sending"
__licence__ = "BSD 3 Clause"
__version__ = "1.3.1"
__build__ = "2026031001"
__compat__ = "python2.7+"

import logging
import os
import re
import smtplib
import socket
import ssl
import sys
from email import encoders
from email.header import Header
from email.mime.base import MIMEBase
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.utils import formatdate, make_msgid

# Python 2.7 compat fixes
try:
    from typing import Union, List, Optional
except ImportError:
    pass

try:
    SMTPNotSupportedError = smtplib.SMTPNotSupportedError
except AttributeError:

    class SMTPNotSupportedError(smtplib.SMTPConnectError):
        pass


if sys.version_info[0] < 3:

    class ConnectionError(OSError):
        pass

    class ConnectionRefusedError(ConnectionError):
        pass

    class ConnectionAbortedError(ConnectionError):
        pass

    class ConnectionResetError(ConnectionError):
        pass


logger = logging.getLogger()


class Mailer:
    """
    :param sender_mail:
    :param recipient_mails: Accepts space, comma or semi-colon separated email addresses or list of email addresses
    :param split_mails: When multiple mails exist, shall we create an email per address or an unique one
    :param smtp_server:
    :param smtp_port:
    :param smtp_user:
    :param smtp_password:
    :param security:
    :param subject:
    :param body:
    :param attachment: (str/bytes): Path to file, or inline binary data
    :param filename: (str):  specified filename in case we use inline binary data
    :param html_enabled:
    :param bcc_mails:
    :param priority: (bool) set to true to add a high priority flag
    :param debug:
    :return:
    """

    def __init__(
        self,
        smtp_server="localhost",  # type: str
        smtp_port=25,  # type: int
        smtp_user=None,  # type: str
        smtp_password=None,  # type: str
        security=None,  # type: Optional[str]
        verify_certificates=True,  # type: bool
        hostname=None,  # type: Optional[str]
        encoding="utf-8",  # type: str
        debug=False,  # type: bool
    ):
        # type: (...) -> None
        self.smtp_server = smtp_server
        self.smtp_port = smtp_port
        self.smtp_user = smtp_user
        self.smtp_password = smtp_password
        self.security = security
        self.verify_certificates = verify_certificates
        self.hostname = hostname
        self.encoding = encoding
        self.debug = debug

        # Fix for empty passed auth strings
        if self.smtp_user is not None and len(smtp_user) == 0:
            smtp_user = None
        if smtp_password is not None and len(smtp_password) == 0:
            smtp_password = None

    def send_email(
        self,
        sender_mail=None,  # type: str
        recipient_mails=None,  # type: Union[str, List[str]]
        subject=None,  # type: str
        body=None,  # type: str
        attachment=None,  # type: Optional[Union[str, bytes, List[str], List[bytes]]]
        filename=None,  # type: Optional[Union[str, List[str]]]
        html_enabled=False,  # type: bool
        bcc_mails=None,  # type: str
        priority=False,  # type: bool
        split_mails=False,  # type: bool
    ):
        # (...) -> bool
        if subject is None:
            logger.warning(
                "No subject set for mail from {} to {}.".format(
                    sender_mail, recipient_mails
                )
            )

        if recipient_mails is None:
            raise ValueError("No destination mails set")

        def _send_email(
            recipient_mail,  # type: Union[str,List[str]]
            attachment=None,  # type: Optional[Union[str, bytes, List[str], List[bytes]]]
            filename=None,  # type: Optional[Union[str, List[str]]]
        ):
            # type: (...) -> bool
            """
            Actual mail sending function
            """
            # Create a multipart message and set headers
            message = MIMEMultipart()
            message["From"] = sender_mail
            if isinstance(recipient_mail, list):
                message["To"] = ",".join(recipient_mail)
            else:
                message["To"] = recipient_mail
            message["Date"] = formatdate(localtime=False)
            message["LocalDate"] = formatdate(localtime=True)
            message["Message-Id"] = make_msgid()
            message["Subject"] = Header(subject, self.encoding)
            # Note for myself: Why did we have double subjects ?
            # message["Subject"] = subject

            if bcc_mails is not None:
                message["Bcc"] = bcc_mails  # Recommended for mass emails

            if priority:
                message["X-Priority"] = "2"
                message["X-MSMail-Priority"] = "High"

            # Add body to email
            if body is not None:
                if html_enabled:
                    message.attach(MIMEText(body, "html", self.encoding))
                else:
                    message.attach(MIMEText(body, "plain", self.encoding))

            default_attachment_name = "attachment"
            attachments = []
            filenames = []

            if attachment is not None:
                if isinstance(attachment, list):
                    attachments = attachment
                    if isinstance(filename, list):
                        if len(attachments) != len(filename):
                            raise ValueError(
                                "Mismatch between attachments and filenames list sizes"
                            )
                        filenames = filename
                else:
                    attachments = [attachment]

                if not filenames:
                    for index in range(0, len(attachments)):
                        if isinstance(attachment[index], (bytes, bytearray)):
                            filenames.append(
                                "{}_{}".format(index, default_attachment_name)
                            )
                        elif isinstance(attachment[index], str):
                            filenames.append(os.path.basename(attachment[index]))
                        else:
                            filenames.append(
                                "{}.{}".format(index, default_attachment_name)
                            )

                for attachment, filename in zip(attachments, filenames):
                    if isinstance(attachment, (bytes, bytearray)):
                        payload = attachment
                        if not filename:
                            filename = default_attachment_name
                        filename = filename if filename else default_attachment_name
                    else:
                        with open(attachment, "rb") as f_attachment:
                            payload = f_attachment.read()
                        filename = (
                            filename if filename else os.path.basename(attachment)
                        )

                    part = MIMEBase("application", "octet-stream")
                    part.set_payload(payload)
                    encoders.encode_base64(part)
                    part.add_header(
                        "Content-Disposition", "attachment; filename=%s" % filename
                    )
                    message.attach(part)

            text = message.as_string()

            try:
                if self.security and self.security.lower() in ["ssl", "tls"]:
                    context = ssl.create_default_context()
                    if not self.verify_certificates:
                        context.check_hostname = False
                        context.verify_mode = False

                if self.security and self.security.lower() == "ssl":
                    remote_server = smtplib.SMTP_SSL(
                        self.smtp_server,
                        self.smtp_port,
                    )
                else:
                    # Standard SMTP transaction without security
                    remote_server = smtplib.SMTP(self.smtp_server, self.smtp_port)

                # Now go ahead and play the transaction
                if self.debug:
                    remote_server.set_debuglevel(True)
                remote_server.ehlo(self.hostname)
                if self.security == "tls":
                    remote_server.starttls()
                    remote_server.ehlo(self.hostname)
                if self.smtp_user is not None and self.smtp_password is not None:
                    remote_server.login(self.smtp_user, self.smtp_password)
                remote_server.sendmail(sender_mail, recipient_mail, text)

            # SMTPNotSupportedError = Server does not support STARTTLS
            # SMTPNotSupportedError does not exist in Python 2.x
            except (
                smtplib.SMTPAuthenticationError,
                smtplib.SMTPSenderRefused,
                smtplib.SMTPRecipientsRefused,
                smtplib.SMTPDataError,
                ConnectionRefusedError,
                ConnectionAbortedError,
                ConnectionResetError,
                ConnectionError,
                socket.gaierror,
                socket.error,  # Python 2 error on bad port
                SMTPNotSupportedError,  # Python 2 error when TLS is required
                ssl.SSLError,
            ) as exc:
                logger.error("Cannot send email: %s", exc, exc_info=True)
                return False
            return True

        if not isinstance(recipient_mails, list):
            # Make sure recipient mails is a list
            recipient_mails = re.split(r",|;| ", recipient_mails)

        # Strip extra chars around mail addresses
        recipient_mails = [mail.strip() for mail in recipient_mails]

        rfc822_addresses = [mail for mail in recipient_mails if is_mail_address(mail)]
        non_rfc822_addresses = [
            mail for mail in recipient_mails if mail not in rfc822_addresses
        ]

        result = True

        if split_mails:
            for recipient in rfc822_addresses:
                _result = _send_email(recipient, attachment, filename)
                if not _result:
                    result = _result
        else:
            _result = _send_email(rfc822_addresses, attachment, filename)
            if not _result:
                result = _result

        if not non_rfc822_addresses == []:
            logger.error("Refused non RFC 822 mails: %s", format(non_rfc822_addresses))
            result = False

        return result


def is_mail_address(
    string,  # type: bool
):
    # type: (...) -> bool
    """
    Check email address validity against simpler than RFC822 regex
    """
    if re.match(r"[^@\s]+@[^@\s]+\.[a-zA-Z0-9][a-zA-Z0-9]+$", string):
        return True
    return False


def send_email(
    source_mail=None,  # type: str
    destination_mails=None,  # type: Union[str, List[str]]
    split_mails=False,  # type: bool
    smtp_server="localhost",  # type: str
    smtp_port=25,  # type: int
    smtp_user=None,  # type: str
    smtp_password=None,  # type: str
    security=None,  # type: Optional[str]
    subject=None,  # type: str
    body=None,  # type: str
    attachment=None,  # type: Optional[Union[str, bytes, List[str], List[bytes]]]
    filename=None,  # type: Optional[Union[str, List[str]]]
    html_enabled=False,  # type: bool
    bcc_mails=None,  # type: str
    priority=False,  # type: bool
    debug=False,  # type: bool
):
    # type: (...) -> Union[bool, str]
    """
    Wrapper for compat with earlier ofunctions.mailer
    """

    mailer = Mailer(
        smtp_server=smtp_server,
        smtp_port=smtp_port,
        smtp_user=smtp_user,
        smtp_password=smtp_password,
        security=security,
        debug=debug,
    )

    result = mailer.send_email(
        sender_mail=source_mail,
        recipient_mails=destination_mails,
        split_mails=split_mails,
        subject=subject,
        body=body,
        attachment=attachment,
        filename=filename,
        html_enabled=html_enabled,
        bcc_mails=bcc_mails,
        priority=priority,
    )

    return result



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of ofunctions module

"""
Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes
"""

__intname__ = "tests.ofunctions.process"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2023122901"

import multiprocessing
from datetime import datetime
from time import sleep
import subprocess

from ofunctions.process import *


def test_kill_childs():
    """
    We'll check if kill_childs successfully stops multiprocessing children by checking the execution time
    This test is time based so we don't need to use the child pid logic to test a child pid based function ;)
    """

    def _test_kill_childs(fast_kill):
        workers = 4
        child_exec_time = 120

        process_list = []

        start_time = datetime.utcnow()

        while workers > 0:
            process = multiprocessing.Process(target=sleep, args=(child_exec_time,))
            process.start()
            process_list.append(process)
            workers -= 1

        running_workers = len(process_list)
        print("Running {} workers".format(running_workers))

        childs_still_run = True
        kill_childs_ran = False
        while childs_still_run:
            childs_still_run = False
            for child in process_list:
                if child.is_alive():
                    childs_still_run = True
                # Now let's kill the children if at least 5 seconds elapsed
                cur_exec_time = (datetime.utcnow() - start_time).total_seconds()
                if cur_exec_time >= 5:
                    if not kill_childs_ran:
                        print("Running kill_childs at {} seconds".format(cur_exec_time))
                        kill_childs(fast_kill=fast_kill)
                        kill_childs_ran = True
                sleep(0.1)

        stop_time = datetime.utcnow()

        exec_time = (stop_time - start_time).total_seconds()
        print("Executed workers for {} seconds".format(exec_time))
        assert (
            exec_time < 15
        ), "Execution should have been halted before workers got to finished their job"
        return exec_time

    no_fast_kill_exec_time = _test_kill_childs(fast_kill=False)
    fast_kill_exec_time = _test_kill_childs(fast_kill=True)

    assert (
        no_fast_kill_exec_time > fast_kill_exec_time + 2
    ), "Fast kill should be faster !"


def test_get_processes_by_name():
    """
    Test that we can identify processes by name
    """
    import subprocess

    if os.name == "nt":
        proc_name = "ping.exe"
        exe_path = get_absolute_path(proc_name)
        command = "{} -n 4 127.0.0.1".format(exe_path)
    else:
        proc_name = "ping"
        exe_path = get_absolute_path(proc_name)
        command = ["{}".format(exe_path), "-c", "1", "127.0.0.1"]

    # Don't create process with shell=True or else it will delay creation
    # Create with subprocess.PIPE redir so we don't wait for execution
    created_processes = []
    for i in range(0, 2):
        print("Running command ", command)
        created_processes.append(
            subprocess.Popen(
                command, shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE
            )
        )

    for process in created_processes:
        print("Created process {} with pid {}".format(process, process.pid))

    found_processes = get_processes_by_name(proc_name)

    print(found_processes)

    # Now let's compare, we should have found both processes by now
    for found_process in found_processes:
        found = False
        for created_process in created_processes:
            if found_process.pid == created_process.pid:
                found = True
        if not found:
            assert False, "Did not find process"


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_kill_childs()
    test_get_processes_by_name()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of command_runner module

"""
Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes

"""

__intname__ = "tests.ofunctions.file_utils"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2021052601"

import sys
from time import sleep

from ofunctions.file_utils import *
from ofunctions.random import random_string


def test_check_path_access():
    # logging.basicConfig(level=logging.DEBUG)
    # Hopefully does not exist
    result = check_path_access(
        r"/somedirthathopefullydoesnotexistinthisuniverse2424", check="W"
    )
    assert result is False, "check_path_access failed"
    if os.name == "nt":
        bin_dir = r"C:\Windows\system32"
        tmp_dir = os.path.expandvars("%temp%")
    else:
        bin_dir = "/usr/bin"
        tmp_dir = "/tmp"
    # should be readable
    result = check_path_access(bin_dir, check="R")
    assert result is True, "Access to bin dir {} should be readable".format(bin_dir)
    # should be writable
    check_path_access(tmp_dir, check="W")
    assert result is True, 'Access to current temp "{}" should be writable'.format(
        tmp_dir
    )


def test_glob_path_match():
    """
    Check that "*est*" matches current "tests" dir
    """
    match = glob_path_match(os.path.dirname(__file__), ["*est*"])
    assert match is True, "glob_path_match test failed"


def print_perm_error(file):
    """
    This function solely exists for test_get_paths_recursive
    """
    print("Perm error on: %s" % file)


def test_get_paths_recursive():
    test_directory = os.path.abspath(os.path.dirname(__file__))
    files = get_paths_recursive(test_directory, fn_on_perm_error=print_perm_error)

    assert isinstance(files, chain)
    print('BEGIN FILE LIST IN "{}"'.format(test_directory))
    for file in files:
        print(file)
    print("END FILE LIST")

    files = get_paths_recursive(test_directory)
    result_list = [os.path.basename(file) for file in files]
    assert (
        "test_bisection.py" in result_list
    ), "get_paths_recursive test failed to find test_bisection.py"
    assert (
        len(result_list) > 10
    ), "test dir file list should contain more than 10 entries for ofunctions package"

    # Include directories in output
    files = get_paths_recursive(test_directory, exclude_dirs=False)
    assert "test_json_sanitize.py" in [
        os.path.basename(file) for file in files
    ], "get_paths_recursive with dirs test failed"

    # Try d_exclude_list on ..\tests
    files = get_paths_recursive(
        os.path.join(test_directory, os.pardir),
        d_exclude_list=["tests"],
        exclude_dirs=False,
    )
    assert "tests" + os.sep + "file_utils.py" not in [
        os.path.basename(file) for file in files
    ], "get_paths_recursive with d_exclude_list failed"

    # Try f_exclude_list
    files = get_paths_recursive(test_directory, f_exclude_list=["test_file_utils.py"])
    assert "test_file_utils.py" not in [
        os.path.basename(file) for file in files
    ], "get_paths_recursive with f_exclude_list failed"

    # Try ext_exclude_list
    files = get_paths_recursive(test_directory, ext_exclude_list=[".py"])
    for file in files:
        if file.endswith(".py"):
            assert False, "get_paths_recursive failed with ext_exclude_list"

    # Try f_include_list
    files = get_paths_recursive(
        test_directory, f_include_list=["test_fi*utils.py"], exclude_dirs=True
    )
    result_list = [os.path.basename(file) for file in files]
    assert (
        "test_file_utils.py" in result_list
    ), "get_paths_recursive with f_include_list failed"
    assert len(result_list) == 1, "get_paths_recursive with f_include_list failed"

    # Try ext_include_list
    files = get_paths_recursive(
        test_directory, ext_include_list=[".py"], exclude_dirs=True
    )
    for file in files:
        if not file.endswith(".py"):
            assert False, "get_paths_recursive failed with ext_include_list"

    # Try min_depth & max_depth
    # We should see tests/file_utils.py but not ./__init__.py
    files = get_paths_recursive(
        os.path.join(test_directory, os.pardir),
        min_depth=2,
        max_depth=2,
        exclude_dirs=True,
    )
    result_list = list(files)
    # Let's first check that we don't have a <root_dir>/__init__.py file
    if "__init__.py" in result_list:
        assert (
            False
        ), "get_paths_recursive failed with min & max depth, root __init__.py file found"

    # Now let's check for subdirectory test_file_utils file
    basename_result_list = [os.path.basename(file) for file in result_list]
    if not os.path.normpath("test_file_utils.py") in basename_result_list:
        assert (
            False
        ), "get_paths_recursive failed with min & max depth, file_utils.py not found"


def test_remove_bom():
    utf8_with_bom_data = b"\xef\xbb\xbf\x13\x37\x00\x12\x05\x01\x12\x01\x05"
    utf8_without_bom_data = b"\x13\x37\x00\x12\x05\x01\x12\x01\x05"

    filename = "ofunctions.test_remove_bom." + random_string(16) + ".file"
    remove_file(filename)

    with open(filename, "wb") as fp:
        fp.write(utf8_with_bom_data)

    remove_bom(filename)

    with open(filename, "rb") as fp:
        file_data = fp.read()
    remove_file(filename)

    assert file_data == utf8_without_bom_data, "Test file does not look like it should"


def test_get_file_time():
    for mac_type in ["ctime", "mtime", "atime"]:
        mac_timestamp = get_file_time(__file__, mac_type)
        print(mac_type, mac_timestamp)
        dt = datetime.fromtimestamp(mac_timestamp)
        assert isinstance(
            dt, datetime
        ), "Timestamp could not be converted to datetime object"
        assert 2021 <= dt.year < 2300, "Code will probably not run in 200 years, ehh"


def test_check_file_timestamp_delta():
    """
    Windows file creation dates are VERY wrong when requested by python
    The following code will keep earlier file creation dates, even if file is removed
    Hence we'll add some random string to the filename to make sure the tests will not fail
    """
    filename = (
        "ofunctions.test_check_file_timestamp_delta." + random_string(16) + ".file"
    )
    test_directory = os.path.abspath(os.path.dirname(__file__))
    path = os.path.join(test_directory, filename)
    remove_file(path)

    with open(path, "w") as file_handle:
        file_handle.write("test")
    result = check_file_timestamp_delta(
        path, years=0, days=0, hours=0, minutes=0, seconds=-2
    )
    assert result is False, "Just created file should not be older than 2 seconds"
    sleep(3)
    result = check_file_timestamp_delta(
        path, years=0, days=0, hours=0, minutes=0, seconds=-2
    )
    assert result is True, "Just created file should now be older than 2 seconds"
    remove_file(path)

    result = check_file_timestamp_delta(
        sys.argv[0], years=-200, days=0, hours=0, minutes=0, seconds=0
    )
    assert (
        result is False
    ), "Ahh see... A file older than 200 years ? Is my code still running in the year 2221 ?"


def test_hide_file():
    """
    Dumb checks, need to improve tests here
    We need to verify that attrib has been run on windows and that file begins with dot on unixes
    """
    filename = "ofunctions.test_hide_file." + random_string(16) + ".file"
    test_directory = os.path.abspath(os.path.dirname(__file__))
    path = os.path.join(test_directory, filename)
    remove_file(path)

    with open(path, "w") as fp:
        fp.write("TEST")

    assert hide_file(path), "File is now hidden"
    assert hide_file(path, False), "File is now visible"

    remove_file(path)


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_check_path_access()
    test_glob_path_match()
    test_get_paths_recursive()
    test_remove_bom()
    test_get_file_time()
    test_check_file_timestamp_delta()
    test_hide_file()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of ofunctions module

"""
Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes
"""

__intname__ = "tests.ofunctions.platform"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2023012301"

from ofunctions.platform import *


def test_get_os():
    os = get_os()
    print(os)
    assert os in ["Windows", "Linux"], "Undefined OS"


def test_os_arch():
    arch = os_arch()
    print(arch)
    assert arch in ["x86", "x64"], "Undefined arch"


def test_python_arch():
    arch = python_arch()
    print(arch)
    assert arch in ["x86", "x64"], "Undefined arch"


def test_is_64bit_python():
    is64 = is_64bit_python()
    print(is64)
    assert isinstance(is64, bool), "is_64bit_python() should return a bool"


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_get_os()
    test_python_arch()
    test_is_64bit_python()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of ofunctions module

"""
Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes
"""


__intname__ = "tests.ofunctions.misc"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2023-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2024010201"


from ofunctions.misc import *


def test_bytesconverter():
    x = BytesConverter(1024)
    assert x.bits == 8192, "Bogus BytesConverter result for 1024: {}".format(x.bits)
    assert x.kbytes == 1, "Bogus ByesConverter result for 1024: {}".format(x.kbytes)

    x = BytesConverter("50GB")
    assert x.human == "50.0 GB", "Bogus human conversion for 50GB: {}".format(x.human)
    assert x == 50000000000, "Bogus byte conversion for 50GB: {}".format(x)

    x = BytesConverter("50GiB")
    assert x.human == "53.7 GB", "Bogus human conversion for 50GB: {}".format(x.human)
    assert x == 53687091200, "Bogus byte conversion for 50GB: {}".format(x)

    assert BytesConverter(2049).kbytes == 2
    assert BytesConverter(1000000000000).tbits == 7.2
    assert BytesConverter(4350580).human == "4.4 MB"
    assert BytesConverter("64 KiB") == 65536
    assert BytesConverter("64 KB") == 64000
    assert BytesConverter("64 Kib") == 65536 / 8
    assert BytesConverter("64 Kb") == 64000 / 8

    x = BytesConverter("20MB")
    print(x.human)
    print(x.human_iec_bytes)
    print(x.human_bits)
    print(x.human_iec_bits)
    assert x.human == "20.0 MB"
    assert x.human_iec_bytes == "19.1 MiB"
    assert x.human_bits == "160.0 Mb"
    assert x.human_iec_bits == "152.6 Mib"

    assert BytesConverter(0).mbits == 0
    assert BytesConverter("0 b") == 0
    assert BytesConverter("0 EB") == 0


def test_replace_in_iterable():
    def _fn_key_value(key, value):
        if key == "dic":
            return "--{}--".format(value)
        return value

    def _fn_value(value):
        return "--{}--".format(value)

    dic = {
        "test": {
            "sub": {"dic": 123},
            "other": 456,
        }
    }

    new_dic = replace_in_iterable(dic, _fn_value)
    assert (
        new_dic["test"]["sub"]["dic"] == "--123--"
    ), "fn_value replacement did not succeed"
    assert new_dic["test"]["other"] == "--456--", "fn replacement did not succeed"

    dic = {
        "test": {
            "sub": {"dic": 123},
            "other": 456,
        }
    }

    new_dic = replace_in_iterable(dic, _fn_key_value, callable_wants_key=True)
    assert (
        new_dic["test"]["sub"]["dic"] == "--123--"
    ), "fn_key_value replacement did not succeed"
    assert new_dic["test"]["other"] == 456, "fn replacement did not succeed"


def test_DotDict():
    dic = {"test": {"sub": {"dic": 123}, "li": [123, 456]}}

    dic = DotDict(dic)

    assert dic.test.sub.dic == 123, "Bogus test.sub.dic value"
    assert dic.test.li == [123, 456], "Bogus test.li value"

    dic.test.sub.dic = 456
    assert dic.test.sub.dic == 456, "Setter did not update dict"


def test_fn_name():
    def sub_test_fn_name():
        assert fn_name() == "sub_test_fn_name"
        assert fn_name(1) == "test_fn_name"


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_bytesconverter()
    test_replace_in_iterable()
    test_DotDict()
    test_fn_name()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of ofunctions module

"""
Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes
"""


__intname__ = "tests.ofunctions.service_control"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2021-2025 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2025031301"


import os
from ofunctions.service_control import *


def running_on_github_actions():
    return os.environ.get("RUNNING_ON_GITHUB_ACTIONS") == "true"  # bash 'true'


def test_system_service_handler():
    """
    This checks whether we can start / stop services
    Needs to ne run as administrator in order to succeed

    Assumes the given services are started
    """
    os_name = os.name

    # Let's pick a default service to test (may fail depending on environment when services are not found)
    if os_name == "nt":
        test_service = "themes"
    else:
        test_service = "crond"

    # don't bother to test this on github runner since we don't have privileges, neither can we check if service runs
    if running_on_github_actions():
        return

    status = system_service_handler(test_service, "status")
    print('Service "{}" status {}'.format(test_service, status))
    assert status, '"{}" service is not started'.format(test_service)

    try:
        result = system_service_handler(test_service, "stop")
        assert result, '"{}" service could not be stopped'.format(test_service)
        print('Stopped service "{}": {}'.format(test_service, result))
    except OSError as exc:
        assert (
            False
        ), "Stopping {} service raised an OSError. Are we running as admin ?".format(
            test_service, exc
        )

    try:
        result = system_service_handler(test_service, "start")
        assert result, "{} service could not be started".format(test_service)
        print('Started service "{}": {}'.format(test_service, result))
    except OSError as exc:
        assert (
            False
        ), "Starting {} service raised an OSError. Are we running as admin ?".format(
            test_service, exc
        )


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_system_service_handler()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of ofunctions module

"""
Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes
"""

__intname__ = "tests.ofunctions.threading"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2021-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2024010901"


from re import M
import sys
import os
from time import sleep
from datetime import datetime
from unittest import result
from ofunctions.threading import *


FLOODER_RUNS = 0  # Required global variable so we keep compat with Python 2.7


@threaded
def threaded_sleeper_fn(seconds):
    sleep(seconds)
    return "I slept for {} seconds".format(seconds)

def non_threaded_sleeper_fn(seconds):
    sleep(seconds)
    return "I slept for {} seconds".format(seconds)


def test_threading():
    # Nothing really insane to test here, we just can check whether we get handling back before sleeper_function is finished
    # thread.result() will wait join the thread if not finished

    begin_time = datetime.now()
    thread = threaded_sleeper_fn(2)
    elapsed_time = (datetime.now() - begin_time).total_seconds()
    print("It took {} seconds to run sleeper".format(elapsed_time))

    assert elapsed_time < 1, "We should get handed back control before a second passes"

    if sys.version_info[0] < 3:
        # Python 2.7 cannot use future, hence we won't get a result
        try:
            print(thread.result())
        except AttributeError:
            assert True, "We should get an AttributeError here"
        else:
            assert False, "We didn't get an AttributeError as awaited"
    else:
        result = thread.result()
        assert result == "I slept for 2 seconds", "We should get a proper result"


def test_wait_for_threaded_result():
    begin_time = datetime.now()
    thread = threaded_sleeper_fn(2)
    elapsed_time = (datetime.now() - begin_time).total_seconds()
    assert elapsed_time < 1, "We should get handed back control before a second passes"    
    result = wait_for_threaded_result(thread)
    elapsed_time = (datetime.now() - begin_time).total_seconds()
    print("ELAPSED TIME", elapsed_time)
    assert elapsed_time > 2, "At least 2 seconds should have passed since we got our result back"
    print("RESULT:", result)
    assert result == "I slept for 2 seconds", "Bogus result"


def test_wait_for_threaded_result_timeout():
    begin_time = datetime.now()
    thread = threaded_sleeper_fn(5)
    elapsed_time = (datetime.now() - begin_time).total_seconds()
    assert elapsed_time < 1, "We should get handed back control before a second passes"    
    result = wait_for_threaded_result(thread, timeout=2)
    elapsed_time = (datetime.now() - begin_time).total_seconds()
    print("ELAPSED TIME", elapsed_time)
    assert elapsed_time < 3, "Timeout should have kicked in"
    print("RESULT:", result)
    assert result is None, "Bogus result"

    
def test_wait_for_threaded_result_no_thread():
    begin_time = datetime.now()
    thread = non_threaded_sleeper_fn(2)
    elapsed_time = (datetime.now() - begin_time).total_seconds()
    assert elapsed_time > 2, "At least 2 seconds should have passed since we got our result back"    
    result = wait_for_threaded_result(thread)
    print("RESULT:", result)
    assert result == "I slept for 2 seconds", "Bogus result"

def test_wait_for_threaded_result_list():
    begin_time = datetime.now()
    thread_list = []
    for _ in range(0, 4):
        thread_list.append(threaded_sleeper_fn(2))
    elapsed_time = (datetime.now() - begin_time).total_seconds()
    assert elapsed_time < 1, "We should get handed back control before a second passes"    
    result = wait_for_threaded_result(thread_list)
    elapsed_time = (datetime.now() - begin_time).total_seconds()
    print("ELAPSED TIME", elapsed_time)
    assert elapsed_time > 2, "At least 2 seconds should have passed since we got our result back"
    for i in range(0, 4):
        print("RESULT:", i, result[i])
        assert result[i] == "I slept for 2 seconds", "Bogus list result"

def test_wait_for_threaded_result_list_timeout():
    begin_time = datetime.now()
    thread_list = []
    for i in range(2, 6):
        thread_list.append(threaded_sleeper_fn(i))
    elapsed_time = (datetime.now() - begin_time).total_seconds()
    assert elapsed_time < 1, "We should get handed back control before a second passes"    
    result = wait_for_threaded_result(thread_list, timeout=3.5)
    elapsed_time = (datetime.now() - begin_time).total_seconds()
    print("ELAPSED TIME", elapsed_time)
    assert elapsed_time < 4, "timeout should have kicked in after 3 seconds"
    print("RESULT:", result)
    assert isinstance(result, list), "Result should be a dict"
    assert result[0] == "I slept for 2 seconds", "1st result bogus"
    assert result[1] == "I slept for 3 seconds", "2nd result bogus"
    assert result[2] == None, "3rd result bogus"
    assert result[3] == None, "4th result bogus"

    
def test_wait_for_threaded_result_no_thread_list():
    begin_time = datetime.now()
    thread_list = []
    for _ in range(0, 4):
        thread_list.append(non_threaded_sleeper_fn(2))
    elapsed_time = (datetime.now() - begin_time).total_seconds()
    assert elapsed_time > 8, "At least 8 seconds should have passed since we got our result back"    
    result = wait_for_threaded_result(thread_list)
    assert isinstance(result, list), "Result should be a list"
    for i in range(0, 4):
        print("RESULT:", i, result[i])
        assert result[i] == "I slept for 2 seconds", "Bogus list result"
  

def test_no_flood_without_arguments():
    # We will run function multiple times, but it should only be run once in less than 5 seconds

    global FLOODER_RUNS
    FLOODER_RUNS = 0

    @no_flood(5)
    def flooder_function():
        global FLOODER_RUNS
        FLOODER_RUNS += 1

    for _ in range(0, 5):
        flooder_function()

    assert FLOODER_RUNS == 1, "flooder_function should not have been run more than once"


def test_no_flood_with_arguments():
    # We will run function multiple times, but it should only be run once in less than 5 seconds with the same arguments
    # and run 5 times with different arguments
    print("Running antiflooding tests")

    global FLOODER_RUNS

    @no_flood(2, multiple_instances_diff_args=True)
    def flooder_function_with_diff_args(var=None, var2=None):
        global FLOODER_RUNS
        FLOODER_RUNS += 1

    @no_flood(2, multiple_instances_diff_args=False)
    def flooder_function_without_diff_args(var=None, var2=None):
        global FLOODER_RUNS
        FLOODER_RUNS += 1

    FLOODER_RUNS = 0
    for i in range(0, 5):
        flooder_function_with_diff_args(i)

    assert (
        FLOODER_RUNS == 5
    ), "flooder_function with different arguments should have been run 5 times, not being intercepted by @no_flood since we allow multiple instances with different arguments"

    FLOODER_RUNS = 0
    for i in range(0, 5):
        flooder_function_without_diff_args(i)

    assert (
        FLOODER_RUNS == 1
    ), "flooder_function without different args should not have been run more than once, being intercepted by @no_flood since we don't allow multiple instances even with different arguments"

    sleep(2)
    flooder_function_without_diff_args(i)
    assert (
        FLOODER_RUNS == 2
    ), "After 2 seconds timeout given to decorator, we should be able to execute function again without being intercepted by @no_flood"


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_threading()
    test_wait_for_threaded_result()
    test_wait_for_threaded_result_timeout()
    test_wait_for_threaded_result_no_thread()
    test_wait_for_threaded_result_list()
    test_wait_for_threaded_result_list_timeout()
    test_wait_for_threaded_result_no_thread_list()
    test_no_flood_without_arguments()
    test_no_flood_with_arguments()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of ofunctions module

"""
Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes
"""

__intname__ = "tests.ofunctions.logger_utils"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2022052301"

import logging

from ofunctions.logger_utils import *
from ofunctions.file_utils import get_writable_random_file


def test_worst_logged_level():
    """
    Extrapolate which was worst loglevel called
    """
    logger = logger_get_logger(console=True, debug=True)
    """
    logger.debug('Something logged')
    called_loglevel = logging.DEBUG
    worst_loglevel = get_worst_logger_level(logger)
    assert worst_loglevel == called_loglevel, "Wrong log level caught ({} != {})".format(worst_loglevel, called_loglevel)
    """
    logger.info("Something logged")
    called_loglevel = logging.INFO
    worst_loglevel = get_worst_logger_level(logger)
    assert (
        worst_loglevel == called_loglevel
    ), "Wrong log level caught ({} != {})".format(worst_loglevel, called_loglevel)
    logger.warning("Something logged")
    called_loglevel = logging.WARNING
    worst_loglevel = get_worst_logger_level(logger)
    assert (
        worst_loglevel == called_loglevel
    ), "Wrong log level caught ({} != {})".format(worst_loglevel, called_loglevel)
    logger.error("Something logged")
    called_loglevel = logging.ERROR
    worst_loglevel = get_worst_logger_level(logger)
    assert (
        worst_loglevel == called_loglevel
    ), "Wrong log level caught ({} != {})".format(worst_loglevel, called_loglevel)
    logger.critical("Something logged")
    called_loglevel = logging.CRITICAL
    worst_loglevel = get_worst_logger_level(logger)
    assert (
        worst_loglevel == called_loglevel
    ), "Wrong log level caught ({} != {})".format(worst_loglevel, called_loglevel)

    # back to a info level call, we should still have CRITICAL as worst level
    logger.info("Something logged")
    called_loglevel = logging.CRITICAL
    worst_loglevel = get_worst_logger_level(logger)
    assert (
        worst_loglevel == called_loglevel
    ), "Wrong log level caught ({} != {})".format(worst_loglevel, called_loglevel)


def test_logger_ger_logger():
    """
    Should be tested on Python 2.7
    """
    logger = logger_get_logger(log_file=get_writable_random_file())
    logger.info("Café unicode accent")


def test_logger_nonwritable_logger():
    logger = logger_get_logger(log_file="/abc/def/nonexistent/filepath")
    logger.info("Café unicode accent")


def test_logger_exception():
    logger = logger_get_logger(console=True)
    try:
        # Log some unicode character which should throw an unhandled UnicodedecodeError exception in Python 2.7
        raise Exception("jörn")
    except Exception as exc:
        logger.exception(exc)


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_worst_logged_level()
    test_logger_ger_logger()
    test_logger_exception()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of ofunctions module

"""
Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes
"""

# python 2.7 compat fixes so all strings are considered unicode
from __future__ import unicode_literals

__intname__ = "tests.ofunctions.string_handling"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2022041603"


import sys
import os
from ofunctions.string_handling import *


def running_on_github_actions():
    """
    This is set in github actions workflow with
          env:
        RUNNING_ON_GITHUB_ACTIONS: true
    """
    return os.environ.get("RUNNING_ON_GITHUB_ACTIONS") == "true"  # bash 'true'


def test_string_handling():
    """
    Since we're dealing potentially with python2, let's have unicode strings for comparison
    """
    test_string = 'ABCDxyz1234&é"-è_çà)"ééà()\'"éài^^$¨IÎiï?./%X'

    assert (
        convert_accents(test_string) == 'ABCDxyz1234&e"-e_ca)"eea()\'"eai^^$IIii?./%X'
    ), "Strip accents failed"
    assert (
        strip_special_characters(test_string) == "ABCDxyz1234_iIi.X"
    ), "Strip special chars failed"

    strip_characters_result = 'CDxyz1234&é"-è_çà)"ééà()\'"éài^^$¨IÎiï?./%'
    assert (
        strip_characters(test_string, r"[ABX]") == strip_characters_result
    ), "Should return test_string with A, B and X stripped"
    assert (
        strip_non_alnum_characters(test_string, keep_accents=False) == "ABCDxyz1234iIiX"
    )

    # Python 2 unicode hell on github is worse, just don't worth my time, sorry
    if not running_on_github_actions():
        strip_non_alnum_characters_result = "ABCDxyz1234éèçàééàéàiIÎiïX"
        assert (
            strip_non_alnum_characters(test_string, keep_accents=True)
            == strip_non_alnum_characters_result
        )


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_string_handling()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of ofunctions module

"""
Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes
"""


__intname__ = "tests.ofunctions.mailer"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2025 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2025050201"


from ofunctions.mailer import *


# The following settings need to be overridden by an import file in prod tests
# Of course, we cannot send mails on github actions

SMTP_SERVER = "localhost"
SMTP_PORT = 587
SMTP_USER = ""
SMTP_PASSWORD = ""
SECURITY = "tls"  # or "ssl" or None
SENDER_MAIL = "some@one.tld"
RECIPIENT_MAIL = "single@example.tld"
RECIPIENT_MAILS = ["one@example.tld", "two@example.tld"]
ATTACHMENT = [__file__]
FILENAME = [__file__]
ATTACHMENTS = [b"ABC", b"DEF"]
BODY = "Here is some sleek body"


try:
    from test_mailer_config import *
except ImportError:
    print("No test_mailer_config.py found, using default settings")


def test_mailer():
    mailer = Mailer(smtp_server=SMTP_SERVER, smtp_port=SMTP_PORT, smtp_user=SMTP_USER, smtp_password=SMTP_PASSWORD, security=SECURITY)
    result = mailer.send_email(
        sender_mail=SENDER_MAIL,
        recipient_mails=RECIPIENT_MAIL,
        subject="test test_mailer",
        body=BODY,
    )

    if SMTP_PASSWORD:
        assert result is True, "When set, we should have a valid result"
    else:
        print("We don't really send mails here.")
        assert result is False, "We will not really test this but it sure must fail"


def test_mailer_multi_recipient():
    mailer = Mailer(smtp_server=SMTP_SERVER, smtp_port=SMTP_PORT, smtp_user=SMTP_USER, smtp_password=SMTP_PASSWORD, security=SECURITY)
    result = mailer.send_email(
        sender_mail=SENDER_MAIL,
        recipient_mails=RECIPIENT_MAILS,
        subject="test test_mailer_multi_recipient",
        body=BODY,
    )

    if SMTP_PASSWORD:
        assert result is True, "When set, we should have a valid result"
    else:
        print("We don't really send mails here.")
        assert result is False, "We will not really test this but it sure must fail"


def test_mailer_multi_recipient_split():
    mailer = Mailer(smtp_server=SMTP_SERVER, smtp_port=SMTP_PORT, smtp_user=SMTP_USER, smtp_password=SMTP_PASSWORD, security=SECURITY)
    result = mailer.send_email(
        sender_mail=SENDER_MAIL,
        recipient_mails=RECIPIENT_MAILS,
        subject="test test_mailer_multi_recipient_split",
        body=BODY,
        split_mails=True,
    )
    if SMTP_PASSWORD:
        assert result is True, "When set, we should have a valid result"
    else:
        print("We don't really send mails here.")
        assert result is False, "We will not really test this but it sure must fail"

def test_mailer_single_attachment():
    mailer = Mailer(smtp_server=SMTP_SERVER, smtp_port=SMTP_PORT, smtp_user=SMTP_USER, smtp_password=SMTP_PASSWORD, security=SECURITY)
    result = mailer.send_email(
        sender_mail=SENDER_MAIL,
        recipient_mails=RECIPIENT_MAIL,
        attachment=ATTACHMENT,
        filename=FILENAME,
        subject="test test_mailer_single_attachment",
        body=BODY,
    )
    
    if SMTP_PASSWORD:
        assert result is True, "When set, we should have a valid result"
    else:
        print("We don't really send mails here.")
        assert result is False, "We will not really test this but it sure must fail"  


def test_mailer_multi_attachment():
    mailer = Mailer(smtp_server=SMTP_SERVER, smtp_port=SMTP_PORT, smtp_user=SMTP_USER, smtp_password=SMTP_PASSWORD, security=SECURITY)
    result = mailer.send_email(
        sender_mail=SENDER_MAIL,
        recipient_mails=RECIPIENT_MAIL,
        attachment=ATTACHMENTS,
        subject="test test_mailer_multi_attachment",
        body=BODY,
    )
    if SMTP_PASSWORD:
        assert result is True, "When set, we should have a valid result"
    else:
        print("We don't really send mails here.")
        assert result is False, "We will not really test this but it sure must fail"  


def test_is_mail_address():
    valid = ["me@them.hop", "some@example.org", "herew_we@are.now", "t@td.ff"]
    invalid = ["m@t", "mee@them.t", "s@xlk"]

    for mail in valid:
        assert (
            is_mail_address(mail) is True
        ), "Valid mail address check failed: {} = {}".format(
            mail, is_mail_address(mail)
        )
    for mail in invalid:
        assert (
            is_mail_address(mail) is False
        ), "Invalid mail address check succeed: {} = {}".format(
            mail, is_mail_address(mail)
        )


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_is_mail_address()
    test_mailer()
    test_mailer_multi_recipient()
    test_mailer_multi_recipient_split()
    test_mailer_single_attachment()
    test_mailer_multi_attachment()


#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of ofunctions module

"""
Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes
"""


__intname__ = "tests.ofunctions.random"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2021020901"

from ofunctions.string_handling import accent_chars, ambiguous_chars
from ofunctions.random import *


def test_pw_gen():
    password = pw_gen(20)
    print("Generated password: %s" % password)
    assert len(password) == 20, "Password should be 20 char long"
    for char in password:
        assert (
            char in string.ascii_letters or char in string.digits
        ), "Password is not compliant"


def test_password_gen():
    """
    Since we're doing randum stuff, we need 
    to run the test multiple times to lower error probability
    """
    for i in range(50):
        password = password_gen(20, alpha=True, numeric=True, special_chars=True, no_accents=True, ambiguous_filter=True)
        print("Generated password: %s" % password)
        assert len(password) == 20, "Password should be 20 char long"
        for char in password:
            assert (
                char not in accent_chars and \
                char not in ambiguous_chars  
            ), "Password is not compliant"



if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_pw_gen()
    test_password_gen()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of ofunctions module

"""
Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes
"""

__intname__ = "tests.ofunctions.json_sanitize"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2021020901"


from ofunctions.json_sanitize import *


def test_json_sanitize():
    test_json = {"some.var": "some\nvalue\0without\x00special\tchars"}
    sanitized = json_sanitize(test_json)
    print("Sanitized json: %s" % sanitized)
    for key, value in sanitized.items():
        assert "." not in key, "JSON key should not contain dots"
        assert "\n" not in value, "JSON value should not contain control chars"


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_json_sanitize()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of ofunctions module

"""
Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes
"""

__intname__ = "tests.ofunctions.delayed_keyboardinterrupt"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2022150401"


from time import sleep
from ofunctions.delayed_keyboardinterrupt import *


# TODO: How to write an automatic CTRL+C test without doing a lot of threading stuff ???
def test_delayed_keyboard_interrupt():
    with DelayedKeyboardInterrupt():
        print(
            "This is a manual test where you shouldn't be able to use CTRL+C for 2 seconds"
        )
        sleep(2)
        print("done")


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_delayed_keyboard_interrupt()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of ofunctions package


__intname__ = "ofunctions.setup"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2021 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2021031601"

"""
Namespace packaging here

# Make sure we declare an __init__.py file as namespace holder in the package root containing the following

try:
    __import__('pkg_resources').declare_namespace(__name__)
except ImportError:
    from pkgutil import extend_path
    __path__ = extend_path(__path__, __name__)
"""

import sys
import os
import shutil

import pkg_resources
import setuptools


def _read_file(filename):
    here = os.path.abspath(os.path.dirname(__file__))
    if sys.version_info[0] > 2:
        with open(os.path.join(here, filename), "r", encoding="utf-8") as file_handle:
            return file_handle.read()
    else:
        # With python 2.7, open has no encoding parameter, resulting in TypeError
        # Fix with io.open (slow but works)
        from io import open as io_open

        with io_open(
            os.path.join(here, filename), "r", encoding="utf-8"
        ) as file_handle:
            return file_handle.read()


def get_metadata(package_file):
    """
    Read metadata from package file
    """

    _metadata = {}

    for line in _read_file(package_file).splitlines():
        if line.startswith("__version__") or line.startswith("__description__"):
            delim = "="
            _metadata[line.split(delim)[0].strip().strip("__")] = (
                line.split(delim)[1].strip().strip("'\"")
            )
    return _metadata


def parse_requirements(filename):
    """
    There is a parse_requirements function in pip but it keeps changing import path
    Let's build a simple one
    """
    try:
        requirements_txt = _read_file(filename)
        install_requires = [
            str(requirement)
            for requirement in pkg_resources.parse_requirements(requirements_txt)
        ]
        return install_requires
    except OSError:
        print(
            'WARNING: No requirements.txt file found as "{}". Please check path or create an empty one'.format(
                filename
            )
        )


def clear_package_build_path(package_rel_path):
    """
    We need to clean build path, but setuptools will wait for build/lib/package_name so we need to create that
    """
    build_path = os.path.abspath(os.path.join("build", "lib", package_rel_path))
    try:
        # We need to use shutil.rmtree() instead of os.remove() since the latter implementation
        # produces "WindowsError: [Error 5] Access is denied"
        shutil.rmtree("build")
    except FileNotFoundError:
        print("build path: {} does not exist".format(build_path))
    # Now we need to create the 'build/lib/package/subpackage' path so setuptools won't fail
    os.makedirs(build_path)


#  ######### ACTUAL SCRIPT ENTRY POINT

NAMESPACE_PACKAGE_NAME = "ofunctions"
namespace_package_path = os.path.abspath(NAMESPACE_PACKAGE_NAME)
namespace_package_file = os.path.join(namespace_package_path, "__init__.py")
metadata = get_metadata(namespace_package_file)
requirements = parse_requirements(
    os.path.join(namespace_package_path, "requirements.txt")
)
long_description = _read_file("README.md")

# First lets make sure build path is clean (avoiding namespace package pollution in subpackages)
# Clean build dir before every run so we don't make cumulative wheel files
clear_package_build_path(NAMESPACE_PACKAGE_NAME)

# Generic namespace package
setuptools.setup(
    name=NAMESPACE_PACKAGE_NAME,
    namespace_packages=[NAMESPACE_PACKAGE_NAME],
    packages=setuptools.find_namespace_packages(include=["ofunctions.*"]),
    version=metadata["version"],
    install_requires=requirements,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Topic :: Software Development",
        "Topic :: System",
        "Topic :: System :: Operating System",
        "Topic :: System :: Shells",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Operating System :: POSIX :: Linux",
        "Operating System :: POSIX :: BSD :: FreeBSD",
        "Operating System :: POSIX :: BSD :: NetBSD",
        "Operating System :: POSIX :: BSD :: OpenBSD",
        "Operating System :: Microsoft",
        "Operating System :: Microsoft :: Windows",
        "License :: OSI Approved :: BSD License",
    ],
    description=metadata["description"],
    author="NetInvent - Orsiris de Jong",
    author_email="contact@netinvent.fr",
    url="https://github.com/netinvent/ofunctions",
    keywords=["network", "bisection", "logging"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    python_requires=">=3.5",
    # namespace packages don't work well with zipped eggs
    # ref https://packaging.python.org/guides/packaging-namespace-packages/
    zip_safe=False,
)


for package in setuptools.find_namespace_packages(include=["ofunctions.*"]):
    rel_package_path = package.replace(".", os.sep)
    package_path = os.path.abspath(rel_package_path)
    package_file = os.path.join(package_path, "__init__.py")
    metadata = get_metadata(package_file)
    requirements = parse_requirements(os.path.join(package_path, "requirements.txt"))
    print("\n#### PACKAGE package_path={} package_file={} metadata={} requirements={} ####".format(package, package_file, metadata, requirements))

    # Again, we need to clean build paths between runs
    clear_package_build_path(rel_package_path)

    setuptools.setup(
        name=package.replace(".", "_"),
        namespace_packages=[NAMESPACE_PACKAGE_NAME],
        packages=[package],
        package_data={package: ["__init__.py"]},
        version=metadata["version"],
        install_requires=requirements,
        classifiers=[
            "Development Status :: 5 - Production/Stable",
            "Intended Audience :: Developers",
            "Topic :: Software Development",
            "Topic :: System",
            "Topic :: System :: Operating System",
            "Topic :: System :: Shells",
            "Programming Language :: Python",
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: Implementation :: CPython",
            "Programming Language :: Python :: Implementation :: PyPy",
            "Operating System :: POSIX :: Linux",
            "Operating System :: POSIX :: BSD :: FreeBSD",
            "Operating System :: POSIX :: BSD :: NetBSD",
            "Operating System :: POSIX :: BSD :: OpenBSD",
            "Operating System :: Microsoft",
            "Operating System :: Microsoft :: Windows",
            "License :: OSI Approved :: BSD License",
        ],
        description=metadata["description"],
        author="NetInvent - Orsiris de Jong",
        author_email="contact@netinvent.fr",
        url="https://github.com/netinvent/ofunctions",
        keywords=["network", "bisection", "logging"],
        long_description=long_description,
        long_description_content_type="text/markdown",
        python_requires=">=3.5",
        # namespace packages don't work well with zipped eggs
        # ref https://packaging.python.org/guides/packaging-namespace-packages/
        zip_safe=False,
    )



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of ofunctions module

"""
Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes
"""

__intname__ = "tests.ofunctions.checksums"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2021020901"

from ofunctions.checksums import *
from ofunctions.file_utils import remove_file
from ofunctions.random import random_string


def prepare_temp_file():
    filename = "ofunctions.checksum_file." + random_string(16) + ".file"
    test_directory = os.path.abspath(os.path.dirname(__file__))
    path = os.path.join(test_directory, filename)
    remove_file(path)
    return path


def create_test_file():
    """
    Creates a test file which sha256sum should be 4c77f1bd193cac476cea5af2225e8c0177d5a009390aa6e119c211a00cf325c9
    """
    file = prepare_temp_file()
    with open(file, "wb") as fp:
        fp.write(b"haxx0r3000")
    return file


def test_sha256sum():
    test_file = create_test_file()
    sum = sha256sum(test_file)
    assert (
        sum == "4c77f1bd193cac476cea5af2225e8c0177d5a009390aa6e119c211a00cf325c9"
    ), "Bogus checksum"
    remove_file(test_file)


def test_check_file_hash():
    test_file = create_test_file()
    result = check_file_hash(
        test_file, "4c77f1bd193cac476cea5af2225e8c0177d5a009390aa6e119c211a00cf325c9"
    )
    assert result is True, "Bogus file hash check"
    remove_file(test_file)


def test_create_sha256sum_file():
    test_file = create_test_file()
    filename = os.path.basename(test_file)
    test_directory = os.path.abspath(os.path.dirname(test_file))
    create_sha256sum_file(test_directory)
    sum_file = os.path.join(test_directory, "SHA256SUMS.TXT")
    with open(sum_file, "r") as fp:
        data = fp.read()
        assert (
            "{}  {}".format(
                "4c77f1bd193cac476cea5af2225e8c0177d5a009390aa6e119c211a00cf325c9",
                filename,
            )
            in data
        ), "sha256file does not contain my favorite file"
    remove_file(sum_file)
    remove_file(test_file)


def test_create_manifest_from_dir():
    """
    Does not test prefixes
    """
    manifest_file = prepare_temp_file()
    test_file = create_test_file()
    create_manifest_from_dir(manifest_file, os.path.dirname(manifest_file))
    with open(manifest_file, "r") as fp:
        data = fp.read()
        print(
            "{}  {}".format(
                "4c77f1bd193cac476cea5af2225e8c0177d5a009390aa6e119c211a00cf325c9",
                test_file,
            )
        )
        print(data)
        assert (
            "{}  {}".format(
                "4c77f1bd193cac476cea5af2225e8c0177d5a009390aa6e119c211a00cf325c9",
                test_file,
            )
            in data
        ), "sha256file does not contain my favorite file"
    remove_file(manifest_file)
    remove_file(test_file)


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_sha256sum()
    test_check_file_hash()
    test_create_sha256sum_file()
    test_create_manifest_from_dir()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of ofunctions module

"""
Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes
"""

__intname__ = "tests.ofunctions.csv"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2021070201"

import sys
from ofunctions.csv import *
from ofunctions.file_utils import remove_file, get_writable_random_file

# python 2.7 compat fixes
if sys.version_info[0] < 3:
    from io import open as open


def test_csv_dict_reader():
    file = get_writable_random_file("csv_dict_reader")
    # Prepare a basic CSV file

    with open(file, "wt", encoding="utf-8") as fp:
        # python 2.7 compat fixes
        if sys.version_info[0] < 3:
            csv_content = (
                "SOME;CSV HEADER;WITH COLUMNS\n1;2;3\n#commented;out;line\nA;B;C\n4;5;6"
            )
        else:
            csv_content = (
                "SOME;CSV HEADER;WITH COLUMNS\n1;2;3\n#commented;out;line\nA;B;C\n4;5;6"
            )
        fp.write(csv_content)
        fp.flush()

    data = csv_dict_reader(file, skip_comment_char="#", delimiter=";", fieldnames=None)
    for index, line in enumerate(data):
        print(index, line)
        if index == 0:
            assert line["SOME"] == "1"
            assert line["CSV HEADER"] == "2"
            assert line["WITH COLUMNS"] == "3"
        if index == 1:
            assert line["SOME"] == "A"
            assert line["CSV HEADER"] == "B"
            assert line["WITH COLUMNS"] == "C"
        if index == 2:
            assert line["SOME"] == "4"
            assert line["CSV HEADER"] == "5"
            assert line["WITH COLUMNS"] == "6"
    remove_file(file)


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_csv_dict_reader()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of ofunctions module

"""
Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes
"""


__intname__ = "tests.ofunctions.network"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2023122801"


# Use logging so we se actual output of probe_mtu
import os
from ofunctions.network import *


def running_on_github_actions():
    """
    This is set in github actions workflow with
          env:
        RUNNING_ON_GITHUB_ACTIONS: true
    """
    return os.environ.get("RUNNING_ON_GITHUB_ACTIONS").lower() == "true"


def test_ping():
    # ping does not work on GH
    if running_on_github_actions():
        return None

    result = ping()
    print("Ping multiple default hosts result: %s" % result)
    assert result is True, (
        "Cannot ping. This test may fail if the host" "does not have internet indeed."
    )

    result = ping("example.not.existing")
    print("Ping to not existing host result: %s" % result)
    assert result is False, "Should not be able to ping non existing host"

    result = ping(mtu=1300, do_not_fragment=True)
    print("Ping to default hosts MTU 1300: %s" % result)
    assert result is True, "WAN should probably have more than MTU 1400"

    result = ping(mtu=1600, do_not_fragment=True)
    print("Ping to default hosts MTU 1600: %s" % result)
    assert result is False, "WAN should probably not have more than MTU 1500"

    result = ping("::1")
    print("Ping IPv6 localhost: %s" % result)
    assert result is True, "Localhost IPv6 address should be pingable"

    result = ping("::1", do_not_fragment=True)
    print("Ping IPv6 localhost with no fragment bit should not work")
    assert (
        result is False
    ), "Localhost IPv6 address should not be pingable when do_not_fragment=True"

    result = ping(["example.not.existing", "127.0.0.1"], all_targets_must_succeed=True)
    print(
        "Ping multiple hosts of which one is not working with all_targets_must_succeed=True result: %s"
        % result
    )
    assert (
        result is False
    ), "At least one failing host should make all_targets_must_succeed=True ping fail"

    result = ping(["127.0.0.1", "127.0.0.1"], all_targets_must_succeed=True)
    print("Ping multiple hosts with all_targets_must_succeed=True result: %s" % result)
    assert (
        result is True
    ), "all_targets_must_succeed=True ping with working hosts should always work"

    result = ping(
        ["example.not.existing", "example.not.existing"], all_targets_must_succeed=True
    )
    print(
        "Ping multiple hosts not working hots with all_targets_must_succeed=True result: %s"
        % result
    )
    assert (
        result is False
    ), "Failing hosts should make all_targets_must_succeed=True ping fail"


def test_test_http_internet():
    # Hopefully these addresses don't exist
    result = check_http_internet(
        ["http://example.not.existing"], ["http://192.168.90.256"]
    )
    print("HTTP result: %s" % result)
    assert result is False, "Bogus http check should give negative result"

    # This one should give positive result too
    result = check_http_internet(
        ["http://www.google.com", "http://example.not.existing"]
    )
    print("HTTP result: %s" % result)
    assert result is True, (
        "At least one good result should trigger positive result. This test may fail if the host"
        "does not have internet indeed."
    )

    # This one should give negative result
    # Hopefully these addresses don't exist
    result = check_http_internet(
        ["http://example.not.existing"],
        ["http://192.168.90.256"],
        all_targets_must_succeed=True,
    )
    print("HTTP result: %s" % result)
    assert result is False, "Bogus http check should give negative result"

    # This one should give negative result too
    result = check_http_internet(
        ["http://www.google.com", "http://example.not.existing"],
        ["http://1.1.1.1", "http://192.168.90.256"],
        all_targets_must_succeed=True,
    )
    print("HTTP result: %s" % result)
    assert result is False, (
        "With all_targets_must_succeed=True, this test should fail."
        "This test may fail if the host does not have internet indeed."
    )

    # Make sure http://1.1.1.1 or http://1.0.0.1 (or whatever you want to test) works, at least one of those two
    result = check_http_internet(ip_servers=["http://1.1.1.1", "http://1.0.0.1"])
    print("HTTP result: %s" % result)
    assert result is True, (
        "Cannot check http internet. This test may fail if the host"
        "does not have internet indeed."
    )


def test_get_public_ip():
    result = get_public_ip()
    print("Public IP: {}".format(result))
    assert result is not None, "Cannot get public IP"


def test_probe_mtu():
    # ping does not work on GH
    if running_on_github_actions():
        return None

    result = probe_mtu("127.0.0.1", min=1400, max=9000)
    print("Localhost MTU: %s" % result)
    assert (
        9000 <= result <= 65500
    ), "Localhost MTU should be somewhere between 9000 and 65500, is: {}".format(result)

    result = probe_mtu("1.1.1.1")
    print("Internet MTU: %s" % result)
    assert 1492 < result < 1501, "Internet MTU should be 1492<=mtu<=1500"

    # Should return a ValueError
    try:
        probe_mtu("1.2.3.4.5.6.7.8.")
    except ValueError:
        assert True, "Non IP entry cannot be probed, obviously"
    else:
        assert False, "Non IP entry was probed !"

    # Should return a ValueError
    try:
        probe_mtu("127.0.0.1", min=2)
    except ValueError:
        assert True, "MTU should not be lower than 28"
    else:
        assert False, "MTU lower than 28 did not raise an exception"

    # Should return a ValueError
    try:
        probe_mtu("127.0.0.1", method="TCP")
    except ValueError:
        assert True, "Unknown method"
    else:
        assert False, "Unknown MTU probe method did not raise an exception"


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    logger.addHandler(logging.StreamHandler())
    logger.setLevel(logging.INFO)
    test_ping()
    test_test_http_internet()
    test_get_public_ip()
    test_probe_mtu()
